==> klayout-0.10.tar.gz: klayout-0.10/src/layViewObject.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_layViewObject
#define HDR_layViewObject

#include <list>

#include <QtCore/QPoint>
#include <QtGui/QColor>
#include <QtGui/QWidget>

#include "tlCollection.h"
#include "dbTrans.h"
#include "dbBox.h"
#include "layViewOp.h"

class QMouseEvent;
class QImage;

namespace lay {

class ViewObjectCanvas;
class Bitmap;

/**
 *  @brief A view service 
 *
 *  A view service is providing editing or informational services
 *  on the canvas. 
 *  A service can be "active" by requesting mouse
 *  events through a trigger by "select" or "activate" after
 *  which a "grab_mouse" redirects all mouse events to the
 *  view service.
 *  This is an interface that should/must be implemented
 *  by the super class.
 */

class ViewService
  : public tl::Collectable 
{
public: 
  ViewService (ViewObjectCanvas *canvas = 0);

  /**
   *  @brief Mouse press event handler
   *
   *  This method will be called by the ViewObjectCanvas object to
   *  dispatch mouse press events. First, the objects that grabbed
   *  the mouse will receive the events in the reverse order they
   *  grabbed the mouse (last one first). This loop will terminate
   *  if one of the objects has returned "true". 
   *  If this is not the case, then all objects will receive
   *  the event with "prio" set to "false", unless one of the objects
   *  returns "true".
   *
   *  @param p The point at which the button was pressed
   *  @param buttons A ored combination of ShiftButton etc.
   *  @return True to terminate dispatcher
   */ 
  virtual bool mouse_press_event (const db::DPoint &p, unsigned int buttons, bool prio) { return false; }

  /** 
   *  @brief Mouse move event handler
   *
   *  Analogeous to mouse_press_event (see above).
   */
  virtual bool mouse_move_event (const db::DPoint &p, bool prio) { return false; }

  /** 
   *  @brief Mouse release event handler
   *
   *  Analogeous to mouse_press_event (see above).
   *
   *  @param click True, if the mouse was not moved between press/release
   */
  virtual bool mouse_release_event (const db::DPoint &p, bool click, bool prio) { return false; }

  /**
   *  @brief Accessor to the canvas object pointer
   */
  ViewObjectCanvas *canvas () const 
  {  
    return mp_canvas;
  }

  /**
   *  @brief Do an update
   *
   *  This method is supposed to update the internals,
   *  specifically upon changing of the coordinate system
   */
  virtual void update ()
  {
    //  The default implementation does nothing 
  }

  /**
   *  @brief Notify for a change of hierarchy levels shown
   * 
   *  With this method, the service is informed about a change in the hierarchy levels
   *  shown.
   */   
  virtual void hier_levels (std::pair<unsigned int, unsigned int> l)
  {
    //  .. The default implementation does nothing ..
  }

  /**
   *  @brief This method is called on the service that got activated
   */  
  virtual void activated () { }

  /**
   *  @brief This method is called on the service that got deactivated
   */  
  virtual void deactivated () { }

  /**
   *  @brief This method is called when a drag operation should be cancelled
   */
  virtual void drag_cancel () { }

  /**
   *  @brief Enable or disable a sevice 
   *
   *  If a service is disabled, it will not receive mouse events 
   */
  void mouse_enable (bool en);

private:
  friend class ViewObjectCanvas;

  ViewObjectCanvas *mp_canvas;
  bool m_abs_grab;
  bool m_enabled;
};

/**
 *  @brief The view object 
 *
 *  A view object is an object that is painted onto the
 *  canvas. The object can be "active" by requesting mouse
 *  events through a trigger by "select" or "activate" after
 *  which a "grab_mouse" redirects all mouse events to the
 *  view object.
 *  This is an interface that should/must be implemented
 *  by the super class.
 */

class ViewObject
  : public tl::Collectable
{
public: 
  /**
   *  @brief The constructor
   *
   *  @param canvas The canvas object that the object is shown on.
   *  @param _static True, if the object is in frozen mode initially
   */
  ViewObject (ViewObjectCanvas *canvas = 0, bool _static = true);

  /**
   *  @brief The copy constructor
   *  
   *  This constructor attaches the object to the canvas the original
   *  object was attached to.
   */
  ViewObject (const ViewObject &d);

  /**
   *  @brief The assignment operator
   *  
   *  This operator leaves the object attached to where it was.
   */
  ViewObject &operator= (const ViewObject &d);

  /**
   *  @brief The destructor
   */
  virtual ~ViewObject ();

  /**
   *  @brief Render the object on the bitmaps provided by the canvas.
   *
   *  This method is supposed to repaint the object on the bitmap
   *  objects that the canvas can provide upon request with the bitmap
   *  method.
   */
  virtual void render () { }

  /**
   *  @brief Render the object on the background 
   *
   *  This method is supposed to repaint the background part of the object 
   *  on the QImage provided.
   *  Only by implementing this method, objects can access the background.
   *  Non-static objects cannot paint in this mode currently.
   */
  virtual void render_bg (QImage &image) { }

  /**
   *  @brief Accessor to the canvas object pointer
   */
  ViewObjectCanvas *canvas () const 
  {  
    return mp_canvas;
  }

  /**
   *  @brief Do an update
   *
   *  This method is supposed to update the internals,
   *  specifically upon changing of the coordinate system
   */
  virtual void update ()
  {
    //  The default implementation does nothing 
  }
 
  /**
   *  @brief Set the visibility state of the view object
   *  
   *  Invisible objects are not drawn
   */
  void visible (bool vis);
     
  /**
   *  @brief Tell the visibility state of the view object
   */
  bool is_visible () const
  {
    return m_visible;
  }
     
  /**
   *  @brief This method tells the canvas to update the object on next repaint
   *  
   *  If the object is static, a "touch" is issued on the canvas. Otherwise
   *  just a update is issued, resulting in a repaint
   *  This is the preferred method to tell that an object needs repainting.
   *  It can be called multiple times without performance penalty.
   */
  void redraw ();

  /**
   *  @brief thaw this object
   *
   *  This is a convenience function that avoids having to store a canvas pointer
   */
  void thaw (); 

  /**
   *  @brief freeze this object
   *
   *  This is a convenience function that avoids having to store a canvas pointer
   */
  void freeze ();

private:
  friend class ViewObjectCanvas;

  ViewObjectCanvas *mp_canvas;
  bool m_static;
  bool m_visible;
};

/**
 *  @brief Describes the button state (supposed to be ored)
 */
enum ButtonState {
  ShiftButton = 1,
  ControlButton = 2,
  AltButton = 4,
  LeftButton = 8,
  MidButton = 16,
  RightButton = 32
};

/**
 *  @brief The view object container
 *
 *  The canvas container holds the view objects and 
 *  manages the the mouse event distribution and 
 *  painting.
 */

class ViewObjectCanvas 
  : public QWidget,
    public tl::CollectionMonitor
{
public:
  typedef tl::Collection<ViewObject>::iterator object_iterator;
  typedef tl::Collection<ViewService>::iterator service_iterator;

  /**
   *  @brief ctor
   */
  ViewObjectCanvas (QWidget *view, const char *name);

  /**
   *  @brief dtor
   */
  ~ViewObjectCanvas ();

  /**
   *  @brief Qt mouse move event handler
   */
  void mouseMoveEvent (QMouseEvent *e);

  /**
   *  @brief Qt mouse button press event handler
   */
  void mousePressEvent (QMouseEvent *e);

  /**
   *  @brief Qt mouse button release event handler
   */
  void mouseReleaseEvent (QMouseEvent *e);

  /**
   *  @brief Cancel all drag operations
   */
  void drag_cancel ();

  /**
   *  @brief Bitmap rendering 
   *
   *  This method is supposed to render the object on the bitmaps provided
   *  by the bitmaps() method. 
   *
   *  @param st True, if only static (frozen) objects are to be redrawn
   */
  void do_render (bool st);

  /**
   *  @brief Qt background paint handler
   *
   *  Objects on the background must provide this paint method that paints
   *  into the image directly. The image provided is a 32bit depth image.
   *  
   *  @param img The Qt image object to produce the objects on.
   */
  void do_render_bg (QImage &img);

  /**
   *  @brief Query if any "static" object needs to be redrawn
   *
   *  To reset this flag, call do_render with "st" set to true.
   */
  bool needs_update_static () const
  {
    return m_needs_update_static;
  }

  /**
   *  @brief Query if the background needs to be redrawn
   *
   *  To reset this flag, call do_render_bg 
   */
  bool needs_update_bg () const
  {
    return m_needs_update_bg;
  }

  /**
   *  @brief Puts the object into "static" mode
   */
  void freeze (ViewObject *obj);

  /**
   *  @brief Puts the object into "non-static" mode
   */
  void thaw (ViewObject *obj);

  /**
   *  @brief Marks the canvas for update
   *
   *  Beside issuing an "update" request, the canvas is also
   *  marked for "static-update", which shall redraw all static
   *  objects also.
   */
  void touch ();

  /**
   *  @brief Marks the canvas background for update
   *
   *  Beside issuing an "update" request, the canvas is also
   *  marked for "bg-update", which shall redraw including the background.
   */
  void touch_bg ();

  /**
   *  @brief Grab the mouse for the object
   *
   *  This will put the object into the list of objects interested
   *  into mouse events with high priority (prio = true on mouse_* events).
   *  Objects with dragging semantics (press-drag-release) may request
   *  "absolute" mouse grab. This way, all mouse events are passed to the
   *  object. The grab should then in any case be released on mouse button
   *  release.
   */
  void grab_mouse (ViewService *obj, bool abs);

  /**
   *  @brief Remove the object from the list of objects that grabbed the mouse
   *
   *  This will also remove any absolute grabs if there is no other object
   *  that needs this.
   */
  void ungrab_mouse (ViewService *obj);

  /**
   *  @brief Determine the active service
   */
  lay::ViewService *active_service () const
  {
    return mp_active_service;
  }

  /**
   *  @brief Activate a service
   *
   *  The active service will be the first to receive mouse events  
   *  with prio==true, unless the mouse is grabbed and the event is
   *  taken by the grabber. 
   *  Passing 0 deactivates all services. The service currently active
   *  will get a "deactivated" signal.
   */
  void activate (lay::ViewService *service);

  /**
   *  @brief Services iterator: begin
   */
  service_iterator begin_services () 
  {
    return m_services.begin ();
  }

  /**
   *  @brief Services iterator: end
   */
  service_iterator end_services () 
  {
    return m_services.end ();
  }

  /**
   *  @brief Objects iterator: begin
   */
  object_iterator begin_objects () 
  {
    return m_objects.begin ();
  }

  /**
   *  @brief Objects iterator: end
   */
  object_iterator end_objects () 
  {
    return m_objects.end ();
  }

  /** 
   *  @brief Remaining mouse press event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_press_event (const db::DPoint &p, unsigned int buttons) { }

  /** 
   *  @brief Remaining mouse release event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_release_event (const db::DPoint &p, bool click) { }

  /** 
   *  @brief Remaining mouse move event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_move_event (const db::DPoint &p) { }

  /** 
   *  @brief Canvas property: transformation of database units to screen units
   */
  const db::DMagTrans &trans () const 
  {
    return m_trans;
  }

  /** 
   *  @brief Change the transformation
   */
  void trans (const db::DMagTrans &trans);

  /**
   *  @brief Compute the viewport of the canvas in micron space
   */
  db::DBox viewport () const;

  /**
   *  @brief Background color property: background color of the canvas
   */
  virtual QColor background_color () const
  {
    return QColor (); // default is "invalid"
  }

  /**
   *  @brief Foreground color property: foreground color of the canvas (some "contrast" color to background)
   */
  virtual QColor foreground_color () const
  {
    return QColor (); // default is "invalid"
  }

  /**
   *  @brief Bitmap provider
   *
   *  This method is used by the view objects to obtain the bitmaps
   *  they should paint themselves on. 
   *  A style can be specified that determines how the bitmap will
   *  be displayed. Bitmaps may be shared if the style is identical.
   */
  virtual lay::Bitmap *bitmap (const lay::ViewOp & /*style*/)
  {
    return 0;
  }

private:
  friend class lay::ViewObject;
  friend class lay::ViewService;

  tl::Collection <lay::ViewObject> m_objects;
  tl::Collection <lay::ViewService> m_services;
  std::list<ViewService *> m_grabbed;
  bool m_needs_update_static;
  bool m_needs_update_bg;
  lay::ViewService *mp_active_service;
  db::DMagTrans m_trans;
  QPoint m_mouse_pos;
  QPoint m_mouse_pressed;

  //  tl::CollectionMonitor interface implementation
  void added (tl::CollectionBase *, tl::Collectable *);
  void removed (tl::CollectionBase *, tl::Collectable *);

  void do_mouse_move (const QPoint &qp);
};

} // namespace lay

#endif

==> klayout-0.14.tar.gz: klayout-0.14/src/layViewObject.h <==

/*

  KLayout Layout Viewer
  Copyright (C) 2006-2007 Matthias Koefferlein

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/



#ifndef HDR_layViewObject
#define HDR_layViewObject

#include <list>
#include <vector>
#include <map>

#include <QtCore/QPoint>
#include <QtGui/QColor>
#include <QtGui/QWidget>

#include "tlCollection.h"
#include "dbTrans.h"
#include "dbBox.h"
#include "layViewOp.h"

class QMouseEvent;
class QImage;

namespace lay {

class Viewport;
class ViewObjectWidget;
class ViewObjectCanvas;
class Bitmap;

/**
 *  @brief A view service 
 *
 *  A view service is providing editing or informational services
 *  on the canvas. 
 *  A service can be "active" by requesting mouse
 *  events through a trigger by "select" or "activate" after
 *  which a "grab_mouse" redirects all mouse events to the
 *  view service.
 *  This is an interface that should/must be implemented
 *  by the super class.
 */

class ViewService
  : public tl::Collectable 
{
public: 
  ViewService (ViewObjectWidget *canvas = 0);

  /**
   *  @brief Mouse press event handler
   *
   *  This method will be called by the ViewObjectWidget object to
   *  dispatch mouse press events. First, the objects that grabbed
   *  the mouse will receive the events in the reverse order they
   *  grabbed the mouse (last one first). This loop will terminate
   *  if one of the objects has returned "true". 
   *  If this is not the case, then all objects will receive
   *  the event with "prio" set to "false", unless one of the objects
   *  returns "true".
   *  This event is not sent immediately when the mouse button is pressed
   *  but when a signification movement for the mouse cursor away from the
   *  original position is detected. If the mouse button is released before
   *  that, a mouse_clicked_event is sent rather than a press-move-release
   *  sequence.
   *
   *  @param p The point at which the button was pressed
   *  @param buttons A ored combination of ShiftButton etc.
   *  @return True to terminate dispatcher
   */ 
  virtual bool mouse_press_event (const db::DPoint &p, unsigned int buttons, bool prio) { return false; }

  /** 
   *  @brief Mouse single-click event handler
   *
   *  Analogeous to mouse_press_event (see above), but sent if the mouse was not moved.
   *  The click event is coincident with the release of the mouse button.
   */
  virtual bool mouse_click_event (const db::DPoint &p, unsigned int buttons, bool prio) { return false; }

  /** 
   *  @brief Mouse double-click event handler
   *
   *  Analogeous to mouse_press_event (see above), but sent if a double-click was detected.
   */
  virtual bool mouse_double_click_event (const db::DPoint &p, unsigned int buttons, bool prio) { return false; }

  /** 
   *  @brief Mouse move event handler
   *
   *  Analogeous to mouse_press_event (see above).
   */
  virtual bool mouse_move_event (const db::DPoint &p, bool prio) { return false; }

  /** 
   *  @brief Mouse release event handler
   *
   *  Analogeous to mouse_press_event (see above).
   */
  virtual bool mouse_release_event (const db::DPoint &p, bool prio) { return false; }

  /** 
   *  @brief Wheel event handler
   *
   *  Analogeous to mouse_press_event (see above).
   *
   *  @param delta The rotation angle in eigths of a degree
   *  @param horizonal True, if the horizontal wheel was turned
   *  @param p The position where the mouse currently is at (in micron units)
   */
  virtual bool wheel_event (int delta, bool horizonal, const db::DPoint &p, unsigned int buttons, bool prio) { return false; }

  /**
   *  @brief Do an update
   *
   *  This method is supposed to update the internals,
   *  specifically upon changing of the coordinate system
   */
  virtual void update ()
  {
    //  The default implementation does nothing 
  }
 
  /**
   *  @brief Accessor to the widget pointer
   */
  ViewObjectWidget *widget () const 
  {  
    return mp_widget;
  }

  /**
   *  @brief Notify for a change of hierarchy levels shown
   * 
   *  With this method, the service is informed about a change in the hierarchy levels
   *  shown.
   */   
  virtual void hier_levels (std::pair<unsigned int, unsigned int> l)
  {
    //  .. The default implementation does nothing ..
  }

  /**
   *  @brief This method is called on the service that got activated
   */  
  virtual void activated () { }

  /**
   *  @brief This method is called on the service that got deactivated
   */  
  virtual void deactivated () { }

  /**
   *  @brief This method is called when a drag operation should be cancelled
   */
  virtual void drag_cancel () { }

  /**
   *  @brief Enable or disable a sevice 
   *
   *  If a service is disabled, it will not receive mouse events 
   */
  void mouse_enable (bool en);

private:
  friend class ViewObjectWidget;

  ViewObjectWidget *mp_widget;
  bool m_abs_grab;
  bool m_enabled;
};

/**
 *  @brief The view object 
 *
 *  A view object is an object that is painted onto the
 *  canvas and is contained by the view object widget. 
 *  The object can be "active" by requesting mouse
 *  events through a trigger by "select" or "activate" after
 *  which a "grab_mouse" redirects all mouse events to the
 *  view object.
 *  This is an interface that should/must be implemented
 *  by the super class.
 */

class ViewObject
  : public tl::Collectable
{
public: 
  /**
   *  @brief The constructor
   *
   *  @param widget The widget object that the object is shown on.
   *  @param _static True, if the object is in frozen mode initially
   */
  ViewObject (ViewObjectWidget *widget = 0, bool _static = true);

  /**
   *  @brief The copy constructor
   *  
   *  This constructor attaches the object to the widget the original
   *  object was attached to.
   */
  ViewObject (const ViewObject &d);

  /**
   *  @brief The assignment operator
   *  
   *  This operator leaves the object attached to where it was.
   */
  ViewObject &operator= (const ViewObject &d);

  /**
   *  @brief The destructor
   */
  virtual ~ViewObject ();

  /**
   *  @brief Render the object on the bitmaps provided by the canvas.
   *
   *  This method is supposed to repaint the object on the bitmap
   *  objects that the canvas can provide upon request with the bitmap
   *  method.
   */
  virtual void render (const Viewport &vp, ViewObjectCanvas &canvas) { }

  /**
   *  @brief Render the object on the background 
   *
   *  This method is supposed to repaint the background part of the object 
   *  on the QImage provided.
   *  Only by implementing this method, objects can access the background.
   *  Non-static objects cannot paint in this mode currently.
   */
  virtual void render_bg (const Viewport &vp, ViewObjectCanvas &canvas) { }

  /**
   *  @brief Accessor to the widget object pointer
   */
  ViewObjectWidget *widget () const 
  {  
    //  if the object has been unlinked because the widget was destroyed, 
    //  we may notice this by the collection begin unset.
    return collection () != 0 ? mp_widget : 0;
  }

  /**
   *  @brief Set the visibility state of the view object
   *  
   *  Invisible objects are not drawn
   */
  void visible (bool vis);
     
  /**
   *  @brief Tell the visibility state of the view object
   */
  bool is_visible () const
  {
    return m_visible;
  }
     
  /**
   *  @brief This method tells the widget to update the object on next repaint
   *  
   *  If the object is static, a "touch" is issued on the widget. Otherwise
   *  just a update is issued, resulting in a repaint
   *  This is the preferred method to tell that an object needs repainting.
   *  It can be called multiple times without performance penalty.
   */
  void redraw ();

  /**
   *  @brief thaw this object
   *
   *  This is a convenience function that avoids having to store a widget pointer
   */
  void thaw (); 

  /**
   *  @brief freeze this object
   *
   *  This is a convenience function that avoids having to store a widget pointer
   */
  void freeze ();

private:
  friend class ViewObjectWidget;

  ViewObjectWidget *mp_widget;
  bool m_static;
  bool m_visible;
};

/**
 *  @brief Describes the button state (supposed to be ored)
 */
enum ButtonState {
  ShiftButton = 1,
  ControlButton = 2,
  AltButton = 4,
  LeftButton = 8,
  MidButton = 16,
  RightButton = 32
};

/**
 *  @brief The view object container
 *
 *  The container holds the view objects and 
 *  manages the the mouse event distribution and 
 *  painting.
 */

class ViewObjectWidget 
  : public QWidget,
    public tl::CollectionMonitor
{
public:
  typedef tl::Collection<ViewObject>::iterator object_iterator;
  typedef tl::Collection<ViewService>::iterator service_iterator;

  /**
   *  @brief ctor
   */
  ViewObjectWidget (QWidget *view, const char *name);

  /**
   *  @brief dtor
   */
  ~ViewObjectWidget ();

  /**
   *  @brief Qt mouse move event handler
   */
  void mouseMoveEvent (QMouseEvent *e);

  /**
   *  @brief Qt mouse button press event handler
   */
  void mousePressEvent (QMouseEvent *e);

  /**
   *  @brief Qt mouse button double-click event handler
   */
  void mouseDoubleClickEvent (QMouseEvent *e);

  /**
   *  @brief Qt mouse button release event handler
   */
  void mouseReleaseEvent (QMouseEvent *e);

  /**
   *  @brief Qt mouse wheel event handler
   */
  void wheelEvent (QWheelEvent *e);

  /**
   *  @brief Cancel all drag operations
   */
  void drag_cancel ();

  /**
   *  @brief Bitmap rendering 
   *
   *  This method is supposed to render the object on the bitmaps provided
   *  by the bitmaps() method. 
   *
   *  @param st True, if only static (frozen) objects are to be redrawn
   */
  void do_render (const Viewport &vp, ViewObjectCanvas &canvas, bool st);

  /**
   *  @brief Qt background paint handler
   *
   *  Objects on the background must provide this paint method that paints
   *  into the image directly. The image provided is a 32bit depth image.
   *  
   *  @param img The Qt image object to produce the objects on.
   */
  void do_render_bg (const Viewport &vp, ViewObjectCanvas &canvas);

  /**
   *  @brief Query if any "static" object needs to be redrawn
   *
   *  To reset this flag, call do_render with "st" set to true.
   */
  bool needs_update_static () const
  {
    return m_needs_update_static;
  }

  /**
   *  @brief Query if the background needs to be redrawn
   *
   *  To reset this flag, call do_render_bg 
   */
  bool needs_update_bg () const
  {
    return m_needs_update_bg;
  }

  /**
   *  @brief Puts the object into "static" mode
   */
  void freeze (ViewObject *obj);

  /**
   *  @brief Puts the object into "non-static" mode
   */
  void thaw (ViewObject *obj);

  /**
   *  @brief Marks the widget for update
   *
   *  Beside issuing an "update" request, the widget is also
   *  marked for "static-update", which shall redraw all static
   *  objects also.
   */
  void touch ();

  /**
   *  @brief Marks the widget background for update
   *
   *  Beside issuing an "update" request, the widget is also
   *  marked for "bg-update", which shall redraw including the background.
   */
  void touch_bg ();

  /**
   *  @brief Grab the mouse for the object
   *
   *  This will put the object into the list of objects interested
   *  into mouse events with high priority (prio = true on mouse_* events).
   *  Objects with dragging semantics (press-drag-release) may request
   *  "absolute" mouse grab. This way, all mouse events are passed to the
   *  object. The grab should then in any case be released on mouse button
   *  release.
   */
  void grab_mouse (ViewService *obj, bool abs);

  /**
   *  @brief Remove the object from the list of objects that grabbed the mouse
   *
   *  This will also remove any absolute grabs if there is no other object
   *  that needs this.
   */
  void ungrab_mouse (ViewService *obj);

  /**
   *  @brief Determine the active service
   */
  lay::ViewService *active_service () const
  {
    return mp_active_service;
  }

  /**
   *  @brief Activate a service
   *
   *  The active service will be the first to receive mouse events  
   *  with prio==true, unless the mouse is grabbed and the event is
   *  taken by the grabber. 
   *  Passing 0 deactivates all services. The service currently active
   *  will get a "deactivated" signal.
   */
  void activate (lay::ViewService *service);

  /**
   *  @brief Services iterator: begin
   */
  service_iterator begin_services () 
  {
    return m_services.begin ();
  }

  /**
   *  @brief Services iterator: end
   */
  service_iterator end_services () 
  {
    return m_services.end ();
  }

  /**
   *  @brief Objects iterator: begin
   */
  object_iterator begin_objects () 
  {
    return m_objects.begin ();
  }

  /**
   *  @brief Objects iterator: end
   */
  object_iterator end_objects () 
  {
    return m_objects.end ();
  }

  /** 
   *  @brief Remaining mouse double click event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_double_click_event (const db::DPoint &p, unsigned int buttons) { }

  /** 
   *  @brief Remaining single mouse click event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_click_event (const db::DPoint &p, unsigned int buttons) { }

  /** 
   *  @brief Remaining mouse press event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_press_event (const db::DPoint &p, unsigned int buttons) { }

  /** 
   *  @brief Remaining mouse release event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_release_event (const db::DPoint &p) { }

  /** 
   *  @brief Remaining mouse move event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void mouse_move_event (const db::DPoint &p) { }

  /** 
   *  @brief Remaining wheel event handler
   *
   *  This event handler is called if no ViewObject requested handling
   *  of this event.
   */
  virtual void wheel_event (int delta, bool horizontal, const db::DPoint &p, unsigned int buttons) { }

  /**
   *  @brief Get the transformation for mouse events
   */
  const db::DCplxTrans &mouse_event_trans () const
  {
    return m_trans;
  }

  /**
   *  @brief Get the viewport for mouse events
   */
  db::DBox mouse_event_viewport () const;

protected:
  /**
   *  @brief Set the transformation for mouse events
   */
  void mouse_event_trans (const db::DCplxTrans &trans);

private:
  friend class lay::ViewObject;
  friend class lay::ViewService;

  tl::Collection <lay::ViewObject> m_objects;
  tl::Collection <lay::ViewService> m_services;
  std::list<ViewService *> m_grabbed;
  bool m_needs_update_static;
  bool m_needs_update_bg;
  lay::ViewService *mp_active_service;
  db::DCplxTrans m_trans;
  QPoint m_mouse_pos;
  QPoint m_mouse_pressed;
  bool m_mouse_pressed_state;
  unsigned int m_mouse_buttons;
  bool m_in_mouse_move;

  //  tl::CollectionMonitor interface implementation
  void added (tl::CollectionBase *, tl::Collectable *);
  void removed (tl::CollectionBase *, tl::Collectable *);

  void do_mouse_move (const QPoint &qp);
};

/**
 *  @brief The canvas interface
 *
 *  The canvas provides ways and attributes that allow the view objects to render
 *  themselves. The basic functionality of this interface is to provide bitmaps
 *  for painting. 
 */
class ViewObjectCanvas
{
public:
  /**
   *  The destructor 
   */
  virtual ~ViewObjectCanvas ();

  /**
   *  @brief Background color property: background color of the canvas
   */
  virtual QColor background_color () const = 0;

  /**
   *  @brief Foreground color property: foreground color of the canvas (some "contrast" color to background)
   */
  virtual QColor foreground_color () const = 0;

  /**
   *  @brief Active color property: color of active elements on the canvas (some "contrast" color to background and different from foreground)
   */
  virtual QColor active_color () const = 0;

  /**
   *  @brief Bitmap provider
   *
   *  This method is used by the view objects to obtain the bitmaps
   *  they should paint themselves on. 
   *  A style can be specified that determines how the bitmap will
   *  be displayed. Bitmaps may be shared if the style is identical.
   *  The bitmap index of the ViewOp is not used. 
   */
  lay::Bitmap *bitmap (const lay::ViewOp &style, unsigned int width, unsigned int height);

  /**
   *  @brief Bitmap provider for a sequence of operations on the same bitmap
   *
   *  This method is used by the view objects to obtain the bitmaps
   *  they should paint themselves on. 
   *  A operator vector can be specified that determines how the bitmap will
   *  be displayed. The operators are executed in the order given on the same bitmap.
   *  Bitmaps may be shared if the styles are identical.
   *  The bitmap index of the ViewOp is not used. 
   */
  lay::Bitmap *bitmap (const std::vector<lay::ViewOp> &style, unsigned int width, unsigned int height);

  /**
   *  @brief Return the number of bitmaps stored so far
   */
  unsigned int fg_bitmaps () const
  {
    return mp_fg_bitmaps.size ();
  }

  /**
   *  @brief Return the foreground bitmap with the given index
   *
   *  The returned pointer may be 0. The index must be less than the one returned
   *  by fg_bitmaps ().
   */
  const lay::Bitmap *fg_bitmap (unsigned int i) const
  {
    return mp_fg_bitmaps [i];
  }

  /**
   *  @brief Direct access to the bitmap pointers
   */

  const std::vector <lay::Bitmap *> &fg_bitmap_vector () const
  {
    return mp_fg_bitmaps;
  }

  /**
   *  @brief Return the foreground bitmap style with the given index
   *
   *  The index must be less than the one returned by fg_bitmaps ().
   */
  const lay::ViewOp &fg_style (unsigned int i) const
  {
    return m_fg_view_ops [i];
  }

  /**
   *  @brief Direct access to the styles vector
   */
  const std::vector <lay::ViewOp> &fg_view_op_vector () const
  {
    return m_fg_view_ops;
  }

  /**
   *  @brief Clear the foreground bitmaps and all associated information
   */
  void clear_fg_bitmaps ();

  /**
   *  @brief Sort the bitmaps in the painting order
   */
  void sort_bitmaps ();

  /**
   *  @brief Return the background image
   */
  virtual QImage &bg_image () = 0;

private:
  std::map <lay::ViewOp, unsigned int> m_fg_bitmap_table;
  std::map <std::vector <lay::ViewOp>, unsigned int> m_fgv_bitmap_table;
  std::vector <lay::Bitmap *> mp_fg_bitmaps;
  std::vector <lay::Bitmap *> mp_alloc_bitmaps;
  std::vector <lay::ViewOp> m_fg_view_ops;
};

} // namespace lay

#endif
